Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Sort.Top
Sorts with erased type indices.
This module enables construction of complex data structures on sorts, like maps, sets, etc, e.g.,
let sorts = Set.empty (module Value.Sort.Top)
Since such structures are required to be monomorphic, the sort type index should be removed using the forget function, before a sort could be stored in it.
Note, that the type index is only removed from the meta language (OCaml) type, but is preserved in the value term, so it could be reconstructed (refined) later.