Module Var.Top
Variables with erased sort index.
This module enables construction of complex data structures on variables, e.g., Set.empty (module Theory.Var.Top).
The variables are ordered by their identifiers so that two variables with the same name but different sorts are compared equal.
include Core_kernel.Bin_prot.Binable.S with type t := t
val bin_size_t : t Bin_prot.Size.sizerval bin_write_t : t Bin_prot.Write.writerval bin_read_t : t Bin_prot.Read.readerval __bin_read_t__ : (int -> t) Bin_prot.Read.readerval bin_shape_t : Bin_prot.Shape.tval bin_writer_t : t Bin_prot.Type_class.writerval bin_reader_t : t Bin_prot.Type_class.readerval bin_t : t Bin_prot.Type_class.tinclude Ppx_sexp_conv_lib.Sexpable.S with type t := t
val t_of_sexp : Sexplib0__.Sexp.t -> tval sexp_of_t : t -> Sexplib0__.Sexp.tinclude Base.Comparable.S with type t := t
val (>=) : t -> t -> boolval (<=) : t -> t -> boolval (<>) : t -> t -> boolval equal : t -> t -> boolval compare : t -> t -> intval ascending : t -> t -> intval descending : t -> t -> intval between : t -> low:t -> high:t -> boolval clamp_exn : t -> min:t -> max:t -> tval clamp : t -> min:t -> max:t -> t Base__.Or_error.tval validate_lbound : min:t Base__.Maybe_bound.t -> t Base__.Validate.checkval validate_ubound : max:t Base__.Maybe_bound.t -> t Base__.Validate.checkval validate_bound :
min:t Base__.Maybe_bound.t ->
max:t Base__.Maybe_bound.t ->
t Base__.Validate.check